# Copyright 2018 Matthew Treinish
#
# This file is part of pyopnsense
#
# pyopnsense is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pyopnsense is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pyopnsense. If not, see <http://www.gnu.org/licenses/>.

from six.moves import urllib

from pyopnsense import client


class NetFlowClient(client.OPNClient):
    """A client for interacting with the diagnostics/netflow endpoint.

    :param str api_key: The API key to use for requests
    :param str api_secret: The API secret to use for requests
    :param str base_url: The base API endpoint for the OPNsense deployment
    """
    def status(self):
        """Return the current netflow status.

        :returns: A dict representing the current status of netflow
        :rtype: dict
        """
        return self._get('diagnostics/netflow/status')


class InterfaceClient(client.OPNClient):
    """A client for interacting with the diagnostics/interface endpoint

    :param str api_key: The API key to use for requests
    :param str api_secret: The API secret to use for requests
    :param str base_url: The base API endpoint for the OPNsense deployment
    """
    def get_ndp(self):
        """Get NDP table for router."""
        return self._get('diagnostics/interface/getNdp')

    def get_arp(self):
        """Get ARP table for router."""
        return self._get('diagnostics/interface/getArp')


class NetworkInsightClient(client.OPNClient):
    """A client for interacting with the diagnostics/networkinsight endpoint.

    :param str api_key: The API key to use for requests
    :param str api_secret: The API secret to use for requests
    :param str base_url: The base API endpoint for the OPNsense deployment
    """
    def get_interfaces(self):
        """Return the available interfaces."""
        return self._get('diagnostics/networkinsight/getinterfaces')

    def get_services(self):
        """Return the available services."""
        return self._get('diagnostics/networkinsight/getservices')

    def get_protocols(self):
        """Return the protocols."""
        return self._get('diagnostics/networkinsight/getprotocols')

    def get_timeserie(self):
        """Return the time serie."""
        return self._get('diagnostics/networkinsight/timeserie')


class SystemHealthClient(client.OPNClient):
    """A client for interacting with the diagnostics/systemhealth endpoint.

    :param str api_key: The API key to use for requests
    :param str api_secret: The API secret to use for requests
    :param str base_url: The base API endpoint for the OPNsense deployment
    """
    def get_health_list(self):
        """Return the health list."""
        return self._get('diagnostics/systemhealth/getRRDlist')

    def get_health_data(self, metric, start=0, stop=0, maxitems=1024,
                        inverse=False, details=False):
        """Return the health data."""
        url = ['diagnostics/systemhealth/getSystemHealth']
        url.append(urllib.parse.quote(metric))
        url.append(start)
        url.append(stop)
        url.append(maxitems)
        if inverse:
            url.append('true')
        else:
            url.append('false')
        if details:
            url.append('true')
        else:
            url.append('false')

        return self._get('/'.join(url))



# Copyright 2018 Matthew Treinish
#
# This file is part of pyopnsense
#
# pyopnsense is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pyopnsense is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pyopnsense. If not, see <http://www.gnu.org/licenses/>.


import json

import mock

from pyopnsense import client
from pyopnsense import exceptions
from pyopnsense.tests import base


class TestOPNClient(base.TestCase):

    @mock.patch('requests.get')
    def test_get_success(self, request_mock):
        response_mock = mock.MagicMock()
        response_mock.status_code = 200
        response_mock.text = json.dumps({'a': 'body'})
        request_mock.return_value = response_mock
        opnclient = client.OPNClient('', '', '')
        resp = opnclient._get('fake_url')
        self.assertEqual({'a': 'body'}, resp)
        request_mock.assert_called_once_with(
            '/fake_url', auth=('', ''), timeout=5, verify=False)

    @mock.patch('requests.get')
    def test_get_failures(self, request_mock):
        response_mock = mock.MagicMock()
        response_mock.status_code = 401
        response_mock.text = json.dumps({'a': 'body'})
        request_mock.return_value = response_mock
        opnclient = client.OPNClient('', '', '')
        self.assertRaises(exceptions.APIException, opnclient._get, 'fake_url')
        request_mock.assert_called_once_with(
            '/fake_url', auth=('', ''), timeout=5, verify=False)

    @mock.patch('requests.post')
    def test_post_success(self, request_mock):
        response_mock = mock.MagicMock()
        response_mock.status_code = 200
        response_mock.text = json.dumps({'a': 'body'})
        request_mock.return_value = response_mock
        opnclient = client.OPNClient('', '', '')
        resp = opnclient._post('fake_url', 'body')
        self.assertEqual({'a': 'body'}, resp)
        request_mock.assert_called_once_with(
            '/fake_url', data='body', auth=('', ''), timeout=5, verify=False)

    @mock.patch('requests.post')
    def test_post_failures(self, request_mock):
        response_mock = mock.MagicMock()
        response_mock.status_code = 401
        response_mock.text = json.dumps({'a': 'body'})
        request_mock.return_value = response_mock
        opnclient = client.OPNClient('', '', '')
        self.assertRaises(
            exceptions.APIException, opnclient._post, 'fake_url', 'body')
        request_mock.assert_called_once_with(
            '/fake_url', data='body', auth=('', ''), timeout=5, verify=False)



# Copyright 2018 Matthew Treinish
#
# This file is part of pyopnsense
#
# pyopnsense is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pyopnsense is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pyopnsense. If not, see <http://www.gnu.org/licenses/>.

import json

from pyopnsense import client


class FirmwareClient(client.OPNClient):
    """A client for interacting with the core/firmware endpoint.

    :param str api_key: The API key to use for requests
    :param str api_secret: The API secret to use for requests
    :param str base_url: The base API endpoint for the OPNsense deployment
    """

    def status(self):
        """Return the current firmware update status.

        :returns: A dict representing the current upgrade status for the
                  OPNsense firmware.
        :rtype: dict
        """
        return self._get('core/firmware/status')

    def upgrade(self, upgrade_list=None):
        """Issue an upgrade request.

        :param list upgrade_list: The list of packages to upgrade. If none are
            specified it will issue a request to upgrade all packages.
        """
        if upgrade_list is None:
            upgrade_list = 'all'
        body = json.dumps({'upgrade': upgrade_list})
        return self._post('core/firmware/upgrade', body)



# Copyright 2018 Matthew Treinish
#
# This file is part of pyopnsense
#
# pyopnsense is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pyopnsense is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pyopnsense. If not, see <http://www.gnu.org/licenses/>.


import json

import requests

from pyopnsense import exceptions

DEFAULT_TIMEOUT = 5

# All the successful HTTP status codes from RFC 7231 & 4918
HTTP_SUCCESS = (200, 201, 202, 203, 204, 205, 206, 207)


class OPNClient(object):
    """Representation of the OPNsense API client."""

    def __init__(self, api_key, api_secret, base_url, verify_cert=False):
        """Initialize the OPNsense API client."""
        self.api_key = api_key
        self.api_secret = api_secret
        self.base_url = base_url
        self.verify_cert = verify_cert

    def _process_response(self, response):
        """Handle the response."""
        if response.status_code in HTTP_SUCCESS:
            return json.loads(response.text)
        else:
            raise exceptions.APIException(
                status_code=response.status_code, resp_body=response.text)

    def _get(self, endpoint):
        req_url = '{}/{}'.format(self.base_url, endpoint)
        response = requests.get(req_url, verify=self.verify_cert,
                                auth=(self.api_key, self.api_secret),
                                timeout=DEFAULT_TIMEOUT)
        return self._process_response(response)

    def _post(self, endpoint, body):
        req_url = '{}/{}'.format(self.base_url, endpoint)
        response = requests.post(req_url, data=body, verify=self.verify_cert,
                                 auth=(self.api_key, self.api_secret),
                                 timeout=DEFAULT_TIMEOUT)
        return self._process_response(response)



#!/usr/bin/env python
# Copyright 2018 Matthew Treinish
#
# This file is part of pyopnsense
#
# pyopnsense is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pyopnsense is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pyopnsense. If not, see <http://www.gnu.org/licenses/>.

import setuptools

# In python < 2.7.4, a lazy loading of package `pbr` will break
# setuptools if some other modules registered functions in `atexit`.
# solution from: http://bugs.python.org/issue15881#msg170215
try:
    import multiprocessing  # noqa
except ImportError:
    pass

setuptools.setup(
    setup_requires=['pbr'],
    pbr=True)
